<?php

namespace Drupal\smart_date_recur\Form;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\datetime\Plugin\Field\FieldWidget\DateTimeWidgetBase;
use Drupal\field\Entity\FieldConfig;
use Drupal\smart_date\Plugin\Field\FieldWidget\SmartDateWidgetBase;
use Drupal\smart_date_recur\Entity\SmartDateOverride;

/**
 * Form controller for Smart Date Recur instace override edit forms.
 *
 * @ingroup smart_date_recur
 */
class SmartDateOverrideForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'smart_date_override_form';
  }

  /**
   * {@inheritdoc}
   */
 public function buildForm(array $form, FormStateInterface $form_state, $rrule = NULL, $index = NULL) {

   // TODO: Show some kind of entity summary? Title at least?
    $instances = $rrule->getRuleInstances();

    // Check for an existing override?

    // Get field config.
    $field_config = FieldConfig::loadByName(
      $rrule->get('entity_type')->getString(),
      $rrule->get('bundle')->getString(),
      $rrule->get('field_name')->getString()
    );
    $defaults = $field_config->getDefaultValueLiteral()[0];

    $values['start'] = DrupalDateTime::createFromTimestamp($instances[$index]['value']);
    $values['end'] = DrupalDateTime::createFromTimestamp($instances[$index]['end_value']);
    $values['duration'] = ($instances[$index]['end_value'] - $instances[$index]['value']) / 60;

    $element = [
      'value' => [
        '#type' => 'datetime',
      ]
    ];
    SmartDateWidgetBase::createWidget($element, $values, $defaults);
    // Add the wrapper class for the Smart Date field so JS and CSS apply.
    $element['#attributes']['class'][] = 'field--type-smartdate';
    $form['override'] = $element;
    $form['#attached']['library'][] = 'smart_date/smart_date';

    // Pass in values to identify the override.
    $form['rrule'] = [
      '#type' => 'hidden',
      '#value' => $rrule->id(),
    ];
    $form['rrule_index'] = [
      '#type' => 'hidden',
      '#value' => $index,
    ];
    if (!empty($instances[$index]['oid'])) {
      $form['oid'] = [
        '#type' => 'hidden',
        '#value' => $instances[$index]['oid'],
      ];
    }

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // TODO: check that end > start
    // TODO: check that values != default
    // if (strlen($form_state->getValue('phone_number')) < 3) {
    //   $form_state->setErrorByName('phone_number', $this->t('The phone number is too short. Please enter a full phone number.'));
    // }

    // $elevate = $form_state->getValue(['elnid', 0]);
    // $exclude = $form_state->getValue(['exnid', 0]);
    //
    // if (!$elevate['target_id'] && !$exclude['target_id']) {
    //   $form_state->setErrorByName('elnid', $this->t('Please specify at least one promoted or excluded node.'));
    // }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    // $oid = $form_state->getValue('oid');
    if ($form_state->getValue('oid')) {
      // Existing override, so update.
    }
    else {
      // Create a new override.
    }
    if (!empty($form_state->getValue('oid'))) {
      // Existing override, so retrieve and update values.
      $override = SmartDateOverride::load($form_state->getValue('oid'));
      // Only the values, end_value, and duration are changeable.
      $override->set('value',       $form_state->getValue('value')->getTimestamp());
      $override->set('end_value',   $form_state->getValue('end_value')->getTimestamp());
      $override->set('duration',    $form_state->getValue('duration'));
    } else {
      $values = [
        'rrule'       => $form_state->getValue('rrule'),
        'rrule_index' => $form_state->getValue('rrule_index'),
        'value'       => $form_state->getValue('value')->getTimestamp(),
        'end_value'   => $form_state->getValue('end_value')->getTimestamp(),
        'duration'    => $form_state->getValue('duration'),
      ];
      // New override, so construct object.
      $override = SmartDateOverride::create($values);
    }
    $override->save();

    // TODO: force refresh of parent entity.
    // TODO: redirect to rrule instance listing.

    // $this->messenger()->addStatus($this->t('Your phone number is @number', ['@number' => $form_state->getValue('phone_number')]));

    // $entity = &$this->entity;
    //
    // $status = parent::save($form, $form_state);
    //
    // switch ($status) {
    //   case SAVED_NEW:
    //     $this->messenger()->addMessage($this->t('Created the %label Search override.', [
    //       '%label' => $entity->label(),
    //     ]));
    //     break;
    //
    //   default:
    //     $this->messenger()->addMessage($this->t('Saved the %label Search override.', [
    //       '%label' => $entity->label(),
    //     ]));
    // }
    $form_state->setRedirect('smart_date_recur.instances', ['rrule' => $form_state->getValue('rrule')]);
  }

}
